@@ -115,12 +115,14 @@ void newtWaitForKey(void);
 void newtClearKeyBuffer(void);
 void newtDelay(unsigned int usecs);
 /* top, left are *not* counting the border */
-int newtOpenWindow(unsigned int left,unsigned int top, 
+int newtOpenWindow(int left,int top, 
 		   unsigned int width,unsigned  int height, 
  		  const char * title);
 int newtCenteredWindow(unsigned int width,unsigned int height, const char * title);
 void newtPopWindow(void);
+void newtPopWindowNoRefresh(void);
 void newtSetColors(struct newtColors colors);
+void newtSetColor(int colorset, char *fg, char *bg);
 void newtRefresh(void);
 void newtSuspend(void);
 void newtSetSuspendCallback(newtSuspendCallback cb, void * data);
@@ -148,6 +150,7 @@ void newtCheckboxSetFlags(newtComponent co, int flags, enum newtFlagsSense sense
 newtComponent newtRadiobutton(int left, int top, const char * text, int isDefault,
 			      newtComponent prevButton);
 newtComponent newtRadioGetCurrent(newtComponent setMember);
+void newtRadioSetCurrent(newtComponent setMember);
 newtComponent newtListitem(int left, int top, const char * text, int isDefault,
 			      newtComponent prevItem, const void * data, int flags);
 void newtListitemSet(newtComponent co, const char * text);
@@ -253,10 +256,20 @@ void newtScaleSet(newtComponent co, unsigned long long amount);
 void newtComponentAddCallback(newtComponent co, newtCallback f, void * data);
 void newtComponentTakesFocus(newtComponent co, int val);
 
+/* This callback is called when a component is destroyed. */
+void newtComponentAddDestroyCallback(newtComponent co,
+				      newtCallback f, void * data);
+
 /* this also destroys all of the components (including other forms) on the 
    form */
 void newtFormDestroy(newtComponent form);	
 
+/* NB: You SHOULD NOT call this for components which have been added
+ * to a form (ie. almost all components).  They are destroyed along
+ * with the form when you call newtFormDestroy.
+ */
+void newtComponentDestroy(newtComponent co);
+
 /* Key codes */
 
 #define NEWT_KEY_TAB			'\t'
@@ -357,7 +370,7 @@ int newtWinMenu(char * title, char * text, int suggestedWidth, int flexDown,
 
 struct newtWinEntry {
     char * text;
-    const char ** value;		/* may be initialized to set default */
+    char ** value;		/* may be initialized to set default */
     int flags;
 };
 
