@@ -28,22 +28,30 @@ void suspend(void * d) {
     newtResume();
 }
 
+void helpCallback(newtComponent co, void * tag) {
+    newtWinMessage("Help", "Ok", tag);
+}
+
 int main(void) {
     newtComponent b1, b2, r1, r2, r3, e2, e3, l1, l2, l3, scale;
-    newtComponent lb, t, rsf, answer;
+    newtComponent lb, t, rsf, answer, timeLabel;
     newtComponent cs[10];
     newtComponent f, chklist, e1;
     struct callbackInfo cbis[3];
     char results[10];
-    char * enr2, * enr3, * scaleVal;
+    const char * enr2, * enr3, *scaleVal;
     void ** selectedList;
     int i, numsel;
     char buf[20];
+    const char * spinner = "-\\|/";
+    const char * spinState;
+    struct newtExitStruct es;
 
     newtInit();
     newtCls();
 
     newtSetSuspendCallback(suspend, NULL);
+    newtSetHelpCallback(helpCallback);
 
     newtDrawRootText(0, 0, "Newt test program");
     newtPushHelpLine(NULL);
@@ -52,13 +60,13 @@ int main(void) {
     newtOpenWindow(2, 2, 30, 10, "first window");
     newtOpenWindow(10, 5, 65, 16, "window 2");
 
-    f = newtForm(NULL, NULL, 0);
+    f = newtForm(NULL, "This is some help text", 0);
     chklist = newtForm(NULL, NULL, 0);
 
     b1 = newtButton(3, 1, "Exit");
     b2 = newtButton(18, 1, "Update");
     r1 = newtRadiobutton(20, 10, "Choice 1", 0, NULL);
-    r2 = newtRadiobutton(20, 11, "Chc 2", 1, r1);
+    r2 = newtRadiobutton(20, 11, "Choice 2", 1, r1);
     r3 = newtRadiobutton(20, 12, "Choice 3", 0, r2);
     rsf = newtForm(NULL, NULL, 0);
     newtFormAddComponents(rsf, r1, r2, r3, NULL);
@@ -75,7 +83,8 @@ int main(void) {
     l3 = newtLabel(3, 8, "Hidden:");
     e1 = newtEntry(12, 6, "", 20, &scaleVal, 0);
     e2 = newtEntry(12, 7, "Default", 20, &enr2, NEWT_FLAG_SCROLL);
-    e3 = newtEntry(12, 8, NULL, 20, &enr3, NEWT_FLAG_HIDDEN);
+/*    e3 = newtEntry(12, 8, NULL, 20, &enr3, NEWT_FLAG_HIDDEN); */
+    e3 = newtEntry(12, 8, NULL, 20, &enr3, NEWT_FLAG_PASSWORD);
 
     cbis[0].state = &results[0];
     cbis[0].en = e1;
@@ -89,7 +98,7 @@ int main(void) {
     newtFormAddComponents(f, rsf, scale, NULL);
 
     lb = newtListbox(45, 1, 6, NEWT_FLAG_MULTIPLE | NEWT_FLAG_BORDER |
-				NEWT_FLAG_SCROLL);
+				NEWT_FLAG_SCROLL | NEWT_FLAG_SHOWCURSOR);
     newtListboxAppendEntry(lb, "First", (void *) 1);
     newtListboxAppendEntry(lb, "Second", (void *) 2);
     newtListboxAppendEntry(lb, "Third", (void *) 3);
@@ -104,30 +113,34 @@ int main(void) {
     newtListboxInsertEntry(lb, "Eleventh", (void *) 11, (void *) 10);
     newtListboxDeleteEntry(lb, (void *) 11);
 
+    spinState = spinner;
+    timeLabel = newtLabel(45, 8, "Spinner: -");
+
     t = newtTextbox(45, 10, 17, 5, NEWT_FLAG_WRAP);
     newtTextboxSetText(t, "This is some text does it look okay?\nThis should be alone.\nThis shouldn't be printed");
 
-    newtFormAddComponents(f, lb, t, NULL);
+    newtFormAddComponents(f, lb, timeLabel, t, NULL);
     newtRefresh();
+    newtFormSetTimer(f, 200);
 
     do {
-	answer = newtRunForm(f);
+	newtFormRun(f, &es);
 
-	if (answer == b2) {
+	if (es.reason == NEWT_EXIT_COMPONENT && es.u.co == b2) {
 	    newtScaleSet(scale, atoi(scaleVal));
 	    newtRefresh();
 	    answer = NULL;
+	} else if (es.reason == NEWT_EXIT_TIMER) {
+	    spinState++;
+	    if (!*spinState) spinState = spinner;
+	    sprintf(buf, "Spinner: %c", *spinState);
+	    newtLabelSetText(timeLabel, buf);
 	}
-    } while (!answer);
-
-    scaleVal = strdup(scaleVal);
-    enr2 = strdup(enr2);
-    enr3 = strdup(enr3);
+    } while (es.reason != NEWT_EXIT_COMPONENT || es.u.co == b2);
 
+    numsel = 0;
     selectedList = newtListboxGetSelection(lb, &numsel);
 
-    newtFormDestroy(f);
-
     newtPopWindow();
     newtPopWindow();
     newtFinished();
@@ -136,11 +149,14 @@ int main(void) {
     printf("got string 2: %s\n", enr2);
     printf("got string 3: %s\n", enr3);
 
+    newtFormDestroy(f);
+
+    printf("\nSelected listbox items (%d):\n", numsel);
     if(selectedList) {
-	printf("\nSelected listbox items:\n");
 	for(i = 0; i < numsel; i++)
-	    puts(selectedList[i]);
+	    printf("#%d\n", (int)(long) selectedList[i]);
     }
+    free(selectedList);
 
     return 0;
 }
