@@ -11,7 +11,7 @@ int main(void) {
     newtComponent button;
     newtComponent form;
     newtComponent answer;
-    void ** result, **ptr;
+    const void **result, **ptr;
     int numselected, i, j;
     int * list;
     
@@ -43,13 +43,14 @@ int main(void) {
     newtCheckboxTreeAddItem(checktree, "Donuts", (void *) 12,
     			    NEWT_FLAG_SELECTED,
     			    NEWT_ARG_APPEND, NEWT_ARG_LAST);
-    newtCheckboxTreeAddItem(checktree, "Bavarian Cream", (void *) 12,
+
+    newtCheckboxTreeAddItem(checktree, "Bavarian Cream", (void *) 301,
     			    NEWT_FLAG_SELECTED,
     			    9, NEWT_ARG_APPEND, NEWT_ARG_LAST);
-    newtCheckboxTreeAddItem(checktree, "Honey dipped", (void *) 12,
+    newtCheckboxTreeAddItem(checktree, "Honey dipped", (void *) 302,
     			    NEWT_FLAG_SELECTED,
     			    9, NEWT_ARG_APPEND, NEWT_ARG_LAST);
-    newtCheckboxTreeAddItem(checktree, "Jelly", (void *) 12,
+    newtCheckboxTreeAddItem(checktree, "Jelly", (void *) 303,
     			    NEWT_FLAG_SELECTED,
     			    9, NEWT_ARG_APPEND, NEWT_ARG_LAST);
 
@@ -87,6 +88,8 @@ int main(void) {
     newtCheckboxTreeAddItem(checktree, "Thirteen", (void *) 213, 0,
     			    1, 1, NEWT_ARG_APPEND, NEWT_ARG_LAST);
 
+    newtCheckboxTreeSetCurrent(checktree, (void *) 12);
+
     button = newtButton(-1, -1, "Exit");
     
     grid = newtCreateGrid(1, 2);
@@ -112,9 +115,11 @@ int main(void) {
     else
 	printf("Current selection (all) (%d):\n", numselected);
     for (i = 0; i < numselected; i++) {
-	j = (int) *ptr++;
+	j = (long) *ptr++;
 	printf("%d\n", j);
     }
+    free(result);
+
     result = newtCheckboxTreeGetMultiSelection(checktree, &numselected, 'b');
     ptr = result;
     if (!result || !numselected)
@@ -122,7 +127,7 @@ int main(void) {
     else
 	printf("Current selection (b) (%d):\n",numselected);
     for (i = 0; i < numselected; i++) {
-	j = (int) *ptr++;
+	j = (long) *ptr++;
 	printf("%d\n", j);
     }
 	
@@ -134,6 +139,7 @@ int main(void) {
     for (i = 0; list && list[i] != NEWT_ARG_LAST; i++)
         printf(" %d", list[i]);
     printf("\n");
+    free(list);
     
     newtFormDestroy(form);
     
