@@ -12,7 +12,6 @@
 struct textbox {
     char ** lines;
     int numLines;
-    char *blankline;
     int linesAlloced;
     int doWrap;
     newtComponent sb_act, sb;
@@ -96,7 +95,8 @@ newtComponent newtTextbox(int left, int top, int width, int height, int flags) {
     tb = malloc(sizeof(*tb));
     co->data = tb;
 
-    if (width < 2) width = 2;
+    if (width < 1)
+	width = 1;
 
     co->ops = &textboxOps;
 
@@ -115,9 +115,6 @@ newtComponent newtTextbox(int left, int top, int width, int height, int flags) {
     tb->topLine = 0;
     tb->textWidth = width;
     tb->isActive = 0;
-    tb->blankline = malloc(width+1);
-    memset(tb->blankline,' ',width);
-    tb->blankline[width] = '\0';
 
     if (flags & NEWT_FLAG_SCROLL) {
 	co->width += 2;
@@ -178,8 +175,12 @@ static void doReflow(const char * text, char ** resultPtr, int width,
     mbstate_t ps;
 
     if (resultPtr) {
-	/* XXX I think this will work */
-	result = malloc(strlen(text) + (strlen(text) / width) + 2);
+	if (width > 1) {
+	    /* use width - 1 for double width characters
+	       that won't fit at end of line */
+	    result = malloc(strlen(text) + (strlen(text) / (width - 1)) + 2);
+	} else
+	    result = malloc(strlen(text) * 2 + 2);
 	*result = '\0';
     }
 	
@@ -193,7 +194,7 @@ static void doReflow(const char * text, char ** resultPtr, int width,
 	    int len;
 		
 	    len = wstrlen(text, end - text);
-	    if (len < width) {
+	    if (len <= width) {
 		if (result) {
 		    strncat(result, text, end - text);
 		    strcat(result, "\n");
@@ -210,11 +211,11 @@ static void doReflow(const char * text, char ** resultPtr, int width,
 		if (*text) text++;
 	    } else {
 		const char *spcptr = NULL;
-	        int spc =0,w2, x;
+	        int spc = 0, w2 = 0, x, w;
 
 	        chptr = text;
-		w2 = 0;
-		for (i = 0; i < width - 1;) {
+		i = 0;
+		while (1) {
 			if ((x=mbrtowc(&tmp,chptr,end-chptr,&ps))<=0)
 				break;
 		        if (spc && !iswspace(tmp))
@@ -224,10 +225,13 @@ static void doReflow(const char * text, char ** resultPtr, int width,
 				spcptr = chptr;
 				w2 = i;
 			}
+			w = wcwidth(tmp);
+			if (w < 0)
+				w = 0;
+			if (i && w + i > width)
+				break;
 			chptr += x;
-			x = wcwidth(tmp);
-			if (x>0)
-			    i+=x;
+			i += w;
 		}
 		howbad += width - w2 + 1;
 #ifdef DEBUG_WRAP		    
@@ -267,6 +271,9 @@ char * newtReflowText(char * text, int width, int flexDown, int flexUp,
     int minbad, minbadwidth, howbad;
     char * expandedText;
 
+    if (width < 1)
+	width = 1;
+
     expandedText = expandTabs(text);
 
     if (flexDown || flexUp) {
@@ -276,7 +283,7 @@ char * newtReflowText(char * text, int width, int flexDown, int flexUp,
 	minbad = -1;
 	minbadwidth = width;
 
-	for (i = min; i <= max; i++) {
+	for (i = min; i >= 1 && i <= max; i++) {
 	    doReflow(expandedText, NULL, i, &howbad, NULL);
 
 	    if (minbad == -1 || howbad < minbad) {
@@ -306,7 +313,7 @@ void newtTextboxSetText(newtComponent co, const char * text) {
 	for (i = 0; i < tb->numLines; i++) 
 	    free(tb->lines[i]);
 	free(tb->lines);
-	tb->linesAlloced = tb->numLines = 0;
+	tb->linesAlloced = tb->numLines = tb->topLine = 0;
     }
 
     expanded = expandTabs(text);
@@ -375,9 +382,7 @@ static void textboxDraw(newtComponent c) {
 
     for (i = 0; (i + tb->topLine) < tb->numLines && i < c->height; i++) {
 	newtGotorc(c->top + i, c->left);
-	SLsmg_write_string(tb->blankline);
-	newtGotorc(c->top + i, c->left);
-	SLsmg_write_string(tb->lines[i + tb->topLine]);
+	SLsmg_write_nstring(tb->lines[i + tb->topLine], tb->textWidth);
     }
     /* put cursor at beginning of text for better accessibility */
     newtGotorc(c->top, c->left);
@@ -469,7 +474,6 @@ static void textboxDestroy(newtComponent co) {
     for (i = 0; i < tb->numLines; i++) 
 	free(tb->lines[i]);
     free(tb->lines);
-    free(tb->blankline);
     free(tb);
     free(co);
 }
