@@ -19,10 +19,14 @@ static void * newtvwindow(char * title, char * button1, char * button2,
     newtGrid grid, buttonGrid;
 
     do {
+	va_list argscopy;
+
+	va_copy(argscopy, args);
 	size += 1000;
 	if (buf) free(buf);
 	buf = malloc(size);
-	i = vsnprintf(buf, size, message, args);
+	i = vsnprintf(buf, size, message, argscopy);
+	va_end(argscopy);
     } while (i >= size || i == -1);
 
     flowedText = newtReflowText(buf, 35, 5, 5, &width, &height);
@@ -153,7 +157,7 @@ int newtWinMenu(char * title, char * text, int suggestedWidth, int flexDown,
     listbox = newtListbox(-1, -1, maxListHeight, 
 		  (needScroll ? NEWT_FLAG_SCROLL : 0) | NEWT_FLAG_RETURNEXIT);
     for (i = 0; items[i]; i++) {
-	newtListboxAddEntry(listbox, items[i], (void *) i);
+	newtListboxAddEntry(listbox, items[i], (void *)(long) i);
     }
 
     newtListboxSetCurrent(listbox, *listItem);
@@ -187,7 +191,8 @@ int newtWinMenu(char * title, char * text, int suggestedWidth, int flexDown,
 
     *listItem = ((long) newtListboxGetCurrent(listbox));
 
-    for (rc = 0; result != buttons[rc] && rc < numButtons; rc++);
+    for (rc = 0; rc < numButtons && result != buttons[rc]; rc++)
+	;
     if (rc == numButtons) 
 	rc = 0; /* F12 or return-on-exit (which are the same for us) */
     else 
@@ -242,7 +247,7 @@ int newtWinEntries(char * title, char * text, int suggestedWidth, int flexDown,
 	newtGridSetField(subgrid, 1, i, NEWT_GRID_COMPONENT,
 		         newtEntry(-1, -1, items[i].value ? 
 				    *items[i].value : NULL, dataWidth,
-				    items[i].value, items[i].flags),
+				    (const char **)items[i].value, items[i].flags),
 		         1, 0, 0, 0, 0, 0);
     }
 
